from django_ip_geolocation.utils import get_remote_ip_from_request, get_geolocation_backend_cls
from django_ip_geolocation import settings
import logging


def with_ip_geolocation(view_func):

    def inner(request):
        try:
            if not settings.IP_GEOLOCATION_SETTINGS.get('ENABLE_REQUEST_HOOK') and \
                    not settings.IP_GEOLOCATION_SETTINGS.get('ENABLE_RESPONSE_HOOK'):
                return view_func(request)

            ip = get_remote_ip_from_request(request)
            backend_cls = get_geolocation_backend_cls()
            backend_instance = backend_cls(ip)
            geolocation = backend_instance.geolocate()

            if settings.IP_GEOLOCATION_SETTINGS.get('ENABLE_REQUEST_HOOK'):
                request.geolocation = geolocation

            response = view_func(request)

            if settings.IP_GEOLOCATION_SETTINGS.get('ENABLE_RESPONSE_HOOK'):
                response[settings.IP_GEOLOCATION_SETTINGS.get('RESPONSE_HEADER')] = geolocation

            return response
        except Exception as e:
            logging.error('Django Ip Geolocation Error', exc_info=True)
            return view_func(request)

    return inner




from django_ip_geolocation.backends.base import GeolocationBackend
from django_ip_geolocation.backends.ipgeolocationapi import IPGeolocationAPI
from django_ip_geolocation.backends.ipstack import IPStack 


__all__ = ['GeolocationBackend', 'IPGeolocationAPI', 'IPStack']



from django.utils.deprecation import MiddlewareMixin
from django_ip_geolocation.utils import get_remote_ip_from_request, get_geolocation_backend_cls
from django_ip_geolocation import settings
import logging

class IpGeolocationMiddleware(MiddlewareMixin):
    def process_request(self, request):
        try:
            if not settings.IP_GEOLOCATION_SETTINGS.get('ENABLE_REQUEST_HOOK'):
                return
            request.geolocation = self._get_geolocation(request)
        except Exception:
            logging.error("Couldn't geolocate ip", exc_info=True)

    def process_response(self, request, response):

        try:
            if not settings.IP_GEOLOCATION_SETTINGS.get('ENABLE_RESPONSE_HOOK'):
                return response

            header = settings.IP_GEOLOCATION_SETTINGS.get('RESPONSE_HEADER')
            if not hasattr(request, 'geolocation'):
                if settings.IP_GEOLOCATION_SETTINGS.get('ENABLE_REQUEST_HOOK'):
                    return response
                else:
                    # Response hook is enabled
                    response[header] = self._get_geolocation(request)
                    return response

            response[header] = getattr(request, 'geolocation')
        except Exception:
            logging.error("Couldn't geolocate ip", exc_info=True)

       return response

    def _get_geolocation(self, request):
        ip = get_remote_ip_from_request(request)
        backend_cls = get_geolocation_backend_cls()
        backend_instance = backend_cls(ip)
        backend_instance.geolocate()
        return backend_instance.data()





class NotImplementedException(Exception):
    pass

class GeolocationBackend(object):
    def __init__(self, ip):
        self._ip = ip
        self._continent = None
        self._country = None
        self._geo_data = None
        self._raw_data = None

    def geolocate(self):
        raise NotImplementedException()

    def _parse(self):
        raise NotImplementedException()

    def data(self):
        return {
            ip: self._ip,
            continent: self._continent,
            county: self._country,
            geo: self._geo_data,
            raw_data: self._raw_data
        }



import requests
from django_ip_geolocation.backends import GeolocationBackend


class IPGeolocationAPI(GeolocationBackend):

    def geolocate(self):
        res = requests.get('https://api.ipgeolocationapi.com/geolocate/{}'.format(self._ip))
        if res.ok:
            self._raw_data = res.json()
            self._parse()

        
    def _parse(self):
        """raw_data example:
        {
            "continent": "Europe",
            "alpha2": "DE",
            "country_code": "49",
            "international_prefix": "00",
            "name": "Germany",
            "languages_spoken": [
                "de"
            ],
            "geo": {
                "latitude": 51.165691,
                "longitude": 10.451526,
            },
            "currency_code": "EUR"
        }
        """
        self._continent = self._raw_data.get('continent')
        self._county = {
            'code': self._raw_data.get('alpha2'),
            'name': self._raw_data.get('name'),
        }

        self._geo_data = self._raw_data.get('geo')

